package com.kingdee.eas.custom.shuiyou.taxCal.service;

import cn.com.servyou.dto.ApiResponse;
import cn.com.servyou.dto.AsyncResult;
import cn.com.servyou.dto.tax.*;
import cn.com.servyou.rmi.client.ClientProxyFactory;
import cn.com.servyou.service.TaxRequest;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.kingdee.bos.BOSException;
import com.kingdee.bos.Context;
import com.kingdee.bos.bsf.service.app.IHRMsfService;
import com.kingdee.bos.dao.ormapping.ObjectUuidPK;
import com.kingdee.eas.common.EASBizException;
import com.kingdee.eas.custom.shuiyou.interfacelog.ILogInfo;
import com.kingdee.eas.custom.shuiyou.interfacelog.LogInfoFactory;
import com.kingdee.eas.custom.shuiyou.interfacelog.LogInfoInfo;
import com.kingdee.eas.custom.shuiyou.task.MessageResult;
import com.kingdee.eas.custom.shuiyou.task.TaskCatalogEnum;
import com.kingdee.eas.custom.shuiyou.uitls.SYUtilsFacadeFactory;
import com.kingdee.eas.custom.shuiyou.utils.ClientProxyFactoryUtils;
import com.kingdee.eas.hr.base.PositionTypeEnum;
import com.kingdee.eas.hr.rec.GenderEnum;
import com.kingdee.eas.util.app.DbUtil;
import com.kingdee.jdbc.rowset.IRowSet;
import com.kingdee.shr.compensation.CardTypeEnum;
import com.kingdee.shr.compensation.EmployedTypeEnum;
import com.kingdee.shr.compensation.PersonStatusEnum;
import com.kingdee.shr.compensation.app.incomeTax.EmploymentSituationEnum;
import com.kingdee.shr.compensation.app.tax.TaxUnitFactory;
import com.kingdee.shr.compensation.app.tax.TaxUnitInfo;
import com.kingdee.shr.compensation.util.Base64Utils;
import com.kingdee.util.StringUtils;
import org.apache.log4j.Logger;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description ύ첽˰
 * @Date 2024/9/2 14:30
 * @Created by Heyuan
 */
public class SubmitAsynTaxCalRequestService implements IHRMsfService {
    private static Logger logger = Logger.getLogger(SubmitAsynTaxCalRequestService.class);

    @Override
    public MessageResult process(Context ctx, Map paramMap) throws EASBizException, BOSException {
        ObjectMapper objectMapper = new ObjectMapper();
        String taskId = null;
        ILogInfo iLogInfo = LogInfoFactory.getLocalInstance(ctx);
        //ӿ־ʵ
        LogInfoInfo logInfo = new LogInfoInfo();
        //
        logInfo.setEntrance(this.getClass().getName());
        logInfo.setBizDate(new Date());
        try {
            logger.info(" CalTaxService,  " + objectMapper.writeValueAsString(paramMap));
            MessageResult messageResult = null;
            String this_taskId = (String) paramMap.get("this_taskId");//id
            String paramData = (String) paramMap.get("paramData");
            Map<String, String> paramDataMap = objectMapper.readValue(paramData, Map.class);
            String taxUnitId = paramDataMap.get("TAXUNITID");//˰λid
            String period = paramDataMap.get("PERIOD");//yyyy-MM
            String creator = paramDataMap.get("creator");//id
            StringBuilder errorMsg = new StringBuilder();
            if (StringUtils.isEmpty(taxUnitId)) {
                errorMsg.append("˰λidΪ!\n");
            }
            if (StringUtils.isEmpty(period)) {
                errorMsg.append("yyyy-MMΪ!\n");
            }
            if (errorMsg.length() > 0) {
                throw new BOSException(errorMsg.toString());
            }
            //ͻ˴
            ClientProxyFactory clientProxyFactory = ClientProxyFactoryUtils.getClientProxyFactory();
            TaxUnitInfo taxUnitInfo = TaxUnitFactory.getLocalInstance(ctx).getTaxUnitInfo(new ObjectUuidPK(taxUnitId));
            //ȡ
            Map<String, List<Map<String, Object>>> caluDataMap = getCaluData(ctx, paramDataMap);
            //˰ӿ
            TaxRequest taxRequest = clientProxyFactory.getTaxRequest();
            //ȡ
            CalculateIndividualIncomeTaxRequest taxRequestPara = getTaxRequestParameter(taxUnitInfo, caluDataMap, period);
            //첽˰ӿ
            ApiResponse<AsyncResult> asyncResultApiResponse = taxRequest.calculateASynIndividualIncomeTax(taxRequestPara);
            String status = asyncResultApiResponse.getHead().getStatus();
            logInfo.setInterfaceAddress("/gateway/iit/calculateTax/calculateASynIndividualIncomeTax");//ӿڵַ
            logInfo.setInterfaceName("calculateASynIndividualIncomeTax");//ӿ
            ObjectMapper mapper = new ObjectMapper();
            logInfo.setInParameter(mapper.writeValueAsString(taxRequestPara));//
            logInfo.setOutParameter(mapper.writeValueAsString(asyncResultApiResponse));//ز
            if ("N".equals(status)) {
                //ʧӦ
                String msg = asyncResultApiResponse.getHead().getMsg();
                messageResult = MessageResult.FAILED(msg);
                logInfo.setErrorInfo(msg);//Ϣ
                logInfo.setStatus("ʧ");
            } else {
                //óɹ
                //id
                String requestId = asyncResultApiResponse.getBody().getRequestId();
                logInfo.setStatus("ɹ");
                paramDataMap.put("requestId", requestId);
                String backParam = mapper.writeValueAsString(paramDataMap);
                // ѯ
                taskId = SYUtilsFacadeFactory.getLocalInstance(ctx).addTask(
                        "getASynIndividualIncomeTaxFeedback",
                        backParam,
                        TaskCatalogEnum.CAL_GET,
                        "");
                messageResult = MessageResult.SUCCESS();
                Map map = new HashMap();
                messageResult.setData(map);
                map.put("taskId", taskId);
                map.put("requestId", requestId);
            }
            iLogInfo.addnew(logInfo);
            return messageResult;
        } catch (Exception e) {
            e.printStackTrace();
            logInfo.setErrorInfo(e.getMessage());//Ϣ
            iLogInfo.addnew(logInfo);
            return MessageResult.ERROR(e.getMessage());
        }
    }

    /**
     * ȡ
     */
    private Map<String, List<Map<String, Object>>> getCaluData(Context ctx, Map<String, String> paramDataMap) throws BOSException {
        try {
            String taxUnitId = paramDataMap.get("TAXUNITID");//˰λid
            String period = paramDataMap.get("PERIOD");//yyyy-MM
            String creator = paramDataMap.get("creator");//id
            String initIds = paramDataMap.get("initIds");//id
            StringBuilder sql = new StringBuilder();
            sql.append("SELECT ").append("\n")
                    .append("item.fnumber AS itemNumber,").append("\n")
                    .append("personRe.FCARDNUMBER,").append("\n")
                    .append("personRe.FCARDTYPE,").append("\n")
                    .append("(CASE WHEN personRe.FREPORTNAME IS NOT NULL THEN personRe.FREPORTNAME ELSE person.fname_l2 END) AS personName,").append("\n")
                    .append("person.fnumber AS personNumber,").append("\n")
                    .append("personRe.FPHONENUMBER,").append("\n")  // ֻ
                    .append("personRe.FISOVERSEA,").append("\n")  // ǷԱ
                    .append("personRe.FGENDER,").append("\n")  // Ա
                    .append("personRe.FBIRTHDAY,").append("\n")  // 
                    .append("personRe.FISDISABILITY,").append("\n")  // Ƿм
                    .append("personRe.FDISCARDNUMBER,").append("\n")  // м֤
                    .append("personRe.FHEROCARDNUMBER,").append("\n")  // ֤
                    .append("personRe.FEMAIL,").append("\n")  // 
                    .append("personRe.FCHINESENAME,").append("\n")  // 
                    .append("personRe.FISHERO,").append("\n")  // Ƿ
                    .append("personRe.FISOLD,").append("\n")  // Ƿ
                    .append("personRe.FADDRESS,").append("\n")  // סʡ
                    .append("personRe.FADDRESSCITY,").append("\n")  // ס
                    .append("personRe.FADDRESSCOUNTY,").append("\n")  // ס
                    .append("personRe.FADDRESSSTREET,").append("\n")  // סֵ
                    .append("personRe.FDESC,").append("\n")  // ע
                    .append("personRe.FHOMETOWN,").append("\n")  // ʡ
                    .append("personRe.FHOMETOWNCITY,").append("\n")  // 
                    .append("personRe.FHOMETOWNCOUNTRY,").append("\n")  // 
                    .append("personRe.FHOMETOWNSTREET,").append("\n")  // ֵ
                    .append("personRe.FHOMETOWNDETAILED,").append("\n")  // ϸַ
                    .append("personRe.FACCOUNT,").append("\n")  // ˺
                    .append("personRe.FOTHERCARDTYPE,").append("\n")  // ֤
                    .append("personRe.FOTHERCARDNUMBER,").append("\n")  // ֤
                    .append("personRe.FTAXREASON,").append("\n")  // ˰
                    .append("personRe.FBANKCARDPROVINCE,").append("\n")  // ʡ
                    .append("bank.fname_l2 AS bankName,").append("\n")  // 
                    .append("personRe.FCONTACTADDRESS,").append("\n")  // ϵַ_ʡ
                    .append("personRe.FCONTACTCITY,").append("\n")  // ϵس
                    .append("personRe.FCONTACTCOUNTRY,").append("\n")  // ϵ
                    .append("personRe.FCONTACTSTREET,").append("\n")  // ϵؽֵ
                    .append("personRe.FCONTACTDETAILED,").append("\n")  // ϵϸַ
                    .append("personRe.FENTRYDATE,").append("\n")  // ״뾳ʱ
                    .append("personRe.FLEAVEDATE,").append("\n")  // Ԥ뾳ʱ
                    .append("personRe.FPOSITION,").append("\n")  // ְ
                    .append("personRe.FINVESTMENTTOTAL,").append("\n")  // Ͷܶ
                    .append("personRe.FINVESTMENTRATE,").append("\n")  // Ͷʱ
                    .append("birthplace.FNAME_L2 AS birthplaceName,").append("\n")  // 
                    .append("nationality.FNAME_L2 AS nationalityName,").append("\n")  // 
                    .append("diploma.fnumber AS diplomaNumber,").append("\n")  // ѧ
                    .append("personReEntry.FPERSONSTATUS,").append("\n")  // Ա״̬
                    .append("personReEntry.FEMPLOYEDDATE,").append("\n")  // ܹ
                    .append("personReEntry.FDEPARTUREDATE,").append("\n")  // ְ
                    .append("personReEntry.FEMPLOYEDTYPE,").append("\n")  // ְܹʹҵ
                    .append("personReEntry.FSITUATION,").append("\n")  // ҵ
                    .append("personReEntry.FDEDUCTFEES,").append("\n")  // Ƿ۳
                    .append("scid.t1,").append("\n")  // 
                    .append("scid.t2,").append("\n")  // ˰
                    .append("scid.t3,").append("\n")  // ϱ
                    .append("scid.t4,").append("\n")  // ҽƱ
                    .append("scid.t5,").append("\n")  // ʧҵ
                    .append("scid.t6,").append("\n")  // ס
                    .append("scid.t12,").append("\n")  // ҵ
                    .append("scid.t13,").append("\n")  // ҵ
                    .append("scid.t14,").append("\n")  // ˰ϱ
                    .append("scid.t16,").append("\n")  // ׼۳ľ
                    .append("scid.t17,").append("\n")  // ˰
                    .append("scid.t18,").append("\n")  // ۳˰
                    .append("scid.t39,").append("\n")  // һԲ
                    .append("scid.t64,").append("\n")  // Ů֧
                    .append("scid.t65,").append("\n")  // ֧
                    .append("scid.t66,").append("\n")  // ݴ֧
                    .append("scid.t67,").append("\n")  // ס֧
                    .append("scid.t68,").append("\n")  // ֧
                    .append("scid.t70 ").append("\n")  // Ӥ׶ջ֧
                    .append("FROM T_HR_STaxCalInitDetail scid ").append("\n")
                    .append("LEFT JOIN T_HR_STaxIncomeItem item ON item.fid = scid.FINCOMEITEMID ").append("\n")
                    .append("LEFT JOIN T_HR_STaxPersonRecordentry personReEntry ON personReEntry.fPersonId = scid.fPersonId AND scid.fTaxUnitId = personReEntry.fTaxUnitId ").append("\n")
                    .append("LEFT JOIN T_HR_STaxPersonRecord personRe ON personReEntry.FBILLID = personRe.fid ").append("\n")
                    .append("LEFT JOIN T_BD_Person person ON person.fid = personRe.FPERSONID ").append("\n")
                    .append("LEFT JOIN T_BD_Nationality birthplace ON birthplace.fid = personRe.FBIRTHPLACEID ").append("\n")
                    .append("LEFT JOIN T_BD_HRDiploma diploma ON diploma.fid = personRe.FEDULEVELID ").append("\n")
                    .append("LEFT JOIN T_HR_PBank bank ON bank.fid = personRe.FHRBANKID ").append("\n")
                    .append("LEFT JOIN T_BD_Nationality nationality ON nationality.fid = personRe.FNATIONALITYID ").append("\n")
                    .append("WHERE scid.FTAXUNITID = '").append(taxUnitId).append("'").append("\n")
                    .append("AND scid.FSTATE = '30' ").append("\n")
                    .append("AND scid.fid in(").append(initIds).append(")").append("\n");
            logger.info("getCaluData.sql:\n" + sql);
            IRowSet iRowSet = DbUtil.executeQuery(ctx, sql.toString());
            Map<String, List<Map<String, Object>>> dataMap = new HashMap<>();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            while (iRowSet.next()) {
                List<Map<String, Object>> list;
                //Ŀ
                String itemNumber = iRowSet.getString("itemNumber");
                if (dataMap.containsKey(itemNumber)) {
                    list = dataMap.get(itemNumber);
                } else {
                    list = new ArrayList<>();
                    dataMap.put(itemNumber, list);
                }
                Map map = new HashMap();
                //
                map.put("personName", iRowSet.getString("personName"));
                //
                map.put("personNumber", iRowSet.getString("personNumber"));
                //֤
                map.put("cardNumber", iRowSet.getString("FCARDNUMBER"));
                //֤
                String cardType = iRowSet.getString("FCARDTYPE");
                if (!StringUtils.isEmpty(cardType)) {
                    CardTypeEnum anEnum = CardTypeEnum.getEnum(cardType);
                    if (anEnum != null) {
                        map.put("cardType", anEnum.getAlias());
                    }
                }
                //Ա״̬
                String personstatus = iRowSet.getString("FPERSONSTATUS");
                if (personstatus != null) {
                    PersonStatusEnum anEnum = PersonStatusEnum.getEnum(personstatus);
                    if (anEnum != null) {
                        map.put("personstatusName", anEnum.getAlias());
                    }
                }
                //ְܹʹҵ
                String employedtype = iRowSet.getString("FEMPLOYEDTYPE");
                if (!StringUtils.isEmpty(employedtype)) {
                    EmployedTypeEnum anEnum = EmployedTypeEnum.getEnum(employedtype);
                    if (anEnum != null) {
                        map.put("employedtypeName", anEnum.getAlias());
                    }
                }
                //ֻ
                map.put("phoneNumber", iRowSet.getString("FPHONENUMBER"));
                //ҵ
                Integer situation = iRowSet.getInt("FSITUATION");
                if (situation != null) {
                    EmploymentSituationEnum anEnum = EmploymentSituationEnum.getEnum(situation);
                    if (anEnum != null) {
                        map.put("situationName", anEnum.getAlias());
                    }
                }
                //Ա
                String gender = iRowSet.getString("FGENDER");
                if (!StringUtils.isEmpty(gender)) {
                    GenderEnum anEnum = GenderEnum.getEnum(gender);
                    if (anEnum != null) {
                        map.put("gender", anEnum.getAlias());
                    }
                }
                //Ա
                String isoversea = iRowSet.getString("FISOVERSEA");
                if ("1".equals(isoversea)) {
                    map.put("isoversea", "");
                } else if ("0".equals(isoversea)) {
                    map.put("isoversea", "");
                }
                //Ƿм
                String isdisability = iRowSet.getString("FISDISABILITY");
                if ("1".equals(isdisability)) {
                    map.put("isdisability", "");
                } else {
                    map.put("isdisability", "");
                }
                //м֤
                map.put("discardnumber", iRowSet.getString("FDISCARDNUMBER"));
                //Ƿ
                String ishero = iRowSet.getString("FISHERO");
                if ("1".equals(ishero)) {
                    map.put("ishero", "");
                } else {
                    map.put("ishero", "");
                }
                //֤
                map.put("herocardnumber", iRowSet.getString("FHEROCARDNUMBER"));
                //Ƿ
                String isold = iRowSet.getString("FISOLD");
                if ("1".equals(isold)) {
                    map.put("isold", "");
                } else {
                    map.put("isold", "");
                }
                //Ƿ۳
                String deductfees = iRowSet.getString("FDEDUCTFEES");
                if ("0".equals(deductfees)) {
                    map.put("deductfees", "");
                } else {
                    map.put("deductfees", "");
                }
                //ְ
                String position = iRowSet.getString("FPOSITION");
                if (!StringUtils.isEmpty(position)) {
                    PositionTypeEnum anEnum = PositionTypeEnum.getEnum(position);
                    if (anEnum != null) {
                        map.put("positionName", anEnum.getAlias());
                    }
                }
                //
                map.put("email", iRowSet.getString("FEMAIL"));
                //Ͷܶ
                map.put("investmenttotal", iRowSet.getBigDecimal("FINVESTMENTTOTAL"));
                //Ͷʱ
                map.put("investmentrate", iRowSet.getBigDecimal("FINVESTMENTRATE"));
                //
                map.put("chinesename", iRowSet.getString("FCHINESENAME"));
                //
                Date birthday = iRowSet.getDate("FBIRTHDAY");
                if (birthday != null) {
                    map.put("birthday", sdf.format(birthday));
                }
                //ܹ
                Date employeddate = iRowSet.getDate("FEMPLOYEDDATE");
                if (employeddate != null) {
                    map.put("employeddate", sdf.format(employeddate));
                }
                //ְ
                Date departuredate = iRowSet.getDate("FDEPARTUREDATE");
                if (departuredate != null) {
                    map.put("departuredate", sdf.format(departuredate));
                }
                //
                map.put("birthplaceName", iRowSet.getString("birthplaceName"));
                //ѧ
                int diplomaNumber = iRowSet.getInt("diplomaNumber");
                //todo ѧ ѧ о
                if (diplomaNumber < 6) {
                    map.put("diploma", "ѧ");
                } else if (diplomaNumber == 6) {
                    map.put("diploma", "ѧ");
                } else if (diplomaNumber > 6) {
                    map.put("diploma", "о");
                }
                //ʡ
                map.put("bankcardprovince", iRowSet.getString("FBANKCARDPROVINCE"));
                //
                map.put("bankName", iRowSet.getString("bankName"));
                //˺
                map.put("account", iRowSet.getString("FACCOUNT"));
                //֤
                map.put("othercardtype", iRowSet.getString("FOTHERCARDTYPE"));
                //֤
                map.put("othercardnumber", iRowSet.getString("FOTHERCARDNUMBER"));
                //סʡ
                map.put("address", iRowSet.getString("FADDRESS"));
                //ס
                map.put("addresscity", iRowSet.getString("FADDRESSCITY"));
                //ס
                map.put("addresscounty", iRowSet.getString("FADDRESSCOUNTY"));
                //סֵ
                map.put("addressstreet", iRowSet.getString("FADDRESSSTREET"));
                //ʡ
                map.put("hometown", iRowSet.getString("FHOMETOWN"));
                //
                map.put("hometowncity", iRowSet.getString("FHOMETOWNCITY"));
                //
                map.put("hometowncountry", iRowSet.getString("FHOMETOWNCOUNTRY"));
                //ֵ
                map.put("hometownstreet", iRowSet.getString("FHOMETOWNSTREET"));
                //ϸַ
                map.put("hometowndetailed", iRowSet.getString("FHOMETOWNDETAILED"));
                //
                map.put("nationalityName", iRowSet.getString("nationalityName"));
                //˰
                map.put("taxreason", iRowSet.getString("FTAXREASON"));
                //״뾳ʱ
                Date entrydate = iRowSet.getDate("FENTRYDATE");
                if (entrydate != null) {
                    map.put("entrydate", sdf.format(entrydate));
                }
                //Ԥ뾳ʱ
                Date leavedate = iRowSet.getDate("FLEAVEDATE");
                if (leavedate != null) {
                    map.put("leavedate", sdf.format(leavedate));
                }
                //ϵַ_ʡ
                map.put("contactaddress", iRowSet.getString("FCONTACTADDRESS"));
                //ϵس
                map.put("contactcity", iRowSet.getString("FCONTACTCITY"));
                //ϵ
                map.put("contactcountry", iRowSet.getString("FCONTACTCOUNTRY"));
                //ϵؽֵ
                map.put("contactstreet", iRowSet.getString("FCONTACTSTREET"));
                //ϵϸַ
                map.put("contactdetailed", iRowSet.getString("FCONTACTDETAILED"));
                //ע
                map.put("desc", iRowSet.getString("FDESC"));
                //
                map.put("t1", iRowSet.getBigDecimal("t1"));
                //˰
                map.put("t2", iRowSet.getBigDecimal("t2"));
                //ϱ
                map.put("t3", iRowSet.getBigDecimal("t3"));
                //ҽƱ
                map.put("t4", iRowSet.getBigDecimal("t4"));
                //ʧҵ
                map.put("t5", iRowSet.getBigDecimal("t5"));
                //ס
                map.put("t6", iRowSet.getBigDecimal("t6"));
                //ҵ
                map.put("t12", iRowSet.getBigDecimal("t12"));
                //ҵ
                map.put("t13", iRowSet.getBigDecimal("t13"));
                //˰ϱ
                map.put("t14", iRowSet.getBigDecimal("t14"));
                //׼۳ľ
                map.put("t16", iRowSet.getBigDecimal("t16"));
                //۳˰
                map.put("t18", iRowSet.getBigDecimal("t18"));
                //һԲ
                map.put("t39", iRowSet.getBigDecimal("t39"));
                //Ů֧
                map.put("t64", iRowSet.getBigDecimal("t64"));
                //֧
                map.put("t65", iRowSet.getBigDecimal("t65"));
                //ݴ֧
                map.put("t66", iRowSet.getBigDecimal("t66"));
                //ס֧
                map.put("t67", iRowSet.getBigDecimal("t67"));
                //֧
                map.put("t68", iRowSet.getBigDecimal("t68"));
                //Ӥ׶ջ֧
                map.put("t70", iRowSet.getBigDecimal("t70"));
                list.add(map);
            }
            return dataMap;
        } catch (Exception e) {
            e.printStackTrace();
            throw new BOSException("ȡݱ" + e.getMessage());
        }
    }


    /**
     * ȡ
     *
     * @param taxUnitInfo
     * @param caluDataMap
     * @param period
     * @return
     * @throws BOSException
     */
    private CalculateIndividualIncomeTaxRequest getTaxRequestParameter(
            TaxUnitInfo taxUnitInfo,
            Map<String, List<Map<String, Object>>> caluDataMap,
            String period) throws BOSException {
        //첽˰
        CalculateIndividualIncomeTaxRequest taxRequestPara = new CalculateIndividualIncomeTaxRequest();
        //id
        String bizNo = UUID.randomUUID().toString();
        //ⲿҵ񶩵*
        taxRequestPara.setBizNo(bizNo);
        //ҵ*
        taxRequestPara.setName(taxUnitInfo.getName());
        //taxRequestPara.setName("ݾͨҽҩ޹˾00");
        //˰*
        taxRequestPara.setTaxNo(taxUnitInfo.getTaxNumber());
        //taxRequestPara.setTaxNo("91310124834931897I");
        //*
        taxRequestPara.setArea(taxUnitInfo.getAreaCode());
        //taxRequestPara.setArea("310101");
        //Ǽ
        taxRequestPara.setRegistrationNumber(taxUnitInfo.getRegNumber());
        //taxRequestPara.setRegistrationNumber("10117310124834931897");
        //ű
        //taxRequestPara.setDeptNo("");
        //
        //taxRequestPara.setDeptName("");
        //˰YYYYMM*
        taxRequestPara.setMonth(period.replace("-", ""));
        //ʵ˺
        //taxRequestPara.setRealNameAccount();
        //ʵ
        //taxRequestPara.setRealNamePassword();
        //ʵ
        //taxRequestPara.setEncryptedRealNamePassword();
        //걨
        //taxRequestPara.setDeclarePassword("Aa123456");
        String declPassword = taxUnitInfo.getDeclPassword();
        if (StringUtils.isEmpty(declPassword)) {
            throw new BOSException("걨벻Ϊ!");
        }
        taxRequestPara.setDeclarePassword(Base64Utils.decode(declPassword));
        //걨
        //taxRequestPara.setEncryptedDeclarePassword();
        //*
        taxRequestPara.setPasswordType("0");
        //ۺ
        //ۺö
        ComplexIncomeRequest complexIncomeRequest = new ComplexIncomeRequest();
        taxRequestPara.setComplexIncomeRequest(complexIncomeRequest);
        //Ƿн
        //complexIncomeRequest.setIsOffLineCalculateTax();
        //ⷽʽ
        //complexIncomeRequest.setDeductionMode();
        //
        //complexIncomeRequest.setRatio();
        //޶
        //complexIncomeRequest.setLimitAmount();
        //н
        NormalSalarySpecIncome normalSalarySpecIncome = getNormalSalarySpecIncome(caluDataMap);
        complexIncomeRequest.setNormalSalarySpecIncome(normalSalarySpecIncome);
        //ȫһԽб
        List<ComplexIncome> annualOneTimeBonusIncomeList = getAnnualOneTimeBonusIncomeList(caluDataMap);
        complexIncomeRequest.setAnnualOneTimeBonusIncome(annualOneTimeBonusIncomeList);
        //һ񱨳б
        List<ComplexIncome> laborRemunerationIncome = getLaborRemunerationIncome(caluDataMap);
        complexIncomeRequest.setLaborRemunerationIncome(laborRemunerationIncome);
        //ͶͬһԲб
        List<ComplexIncome> compensateIncome = getCompensateIncome(caluDataMap);
        complexIncomeRequest.setCompensateIncome(compensateIncome);
        return taxRequestPara;
    }

    /**
     * ͶͬһԲб
     *
     * @param caluDataMap
     * @return
     */
    private List<ComplexIncome> getCompensateIncome(
            Map<String, List<Map<String, Object>>> caluDataMap) {
        List<ComplexIncome> compensateIncome = new ArrayList<ComplexIncome>();
        List<Map<String, Object>> list = caluDataMap.get("jcldhtlb");
        for (int i = 0; list != null && i < list.size(); i++) {
            //˰Ӧ+˰ϸ
            Map<String, Object> map = list.get(i);
            ComplexIncome complexIncome = new ComplexIncome();
            //*
            complexIncome.setName((String) map.get("personName"));
            //֤*
            complexIncome.setLicenseType((String) map.get("cardType"));
            //֤*
            complexIncome.setLicenseNumber((String) map.get("cardNumber"));
            //*  һԲ
            complexIncome.setIncome((BigDecimal) map.get("t39"));
            //˰
            complexIncome.setExemptIncome((BigDecimal) map.get("t2"));
            //
            //complexIncome.setOther(BigDecimal.ZERO);
            //˰      
            complexIncome.setTaxDeduction((BigDecimal) map.get("t17"));
            //ע
            //complexIncome.setRemark("");
            //Ŀ*
            complexIncome.setIncomeItemName("һ񱨳");
            //׼۳ľ      
            complexIncome.setDeductibleDonation((BigDecimal) map.get("t16"));
            compensateIncome.add(complexIncome);
        }
        return compensateIncome;
    }

    /**
     * һ񱨳б
     *
     * @param caluDataMap
     * @return
     */
    private List<ComplexIncome> getLaborRemunerationIncome(Map<String, List<Map<String, Object>>> caluDataMap) {
        List<ComplexIncome> laborRemunerationIncome = new ArrayList<>();
        List<Map<String, Object>> list = caluDataMap.get("lwbclb");
        for (int i = 0; list != null && i < list.size(); i++) {
            Map<String, Object> map = list.get(i);
            ComplexIncome complexIncome = new ComplexIncome();
            //*
            complexIncome.setName((String) map.get("personName"));
            //֤*
            complexIncome.setLicenseType((String) map.get("cardType"));
            //֤*
            complexIncome.setLicenseNumber((String) map.get("cardNumber"));
            //Ƿϸ걨
            //complexIncome.setIsDetailedDeclaration("");
            //*
            complexIncome.setIncome((BigDecimal) map.get("t1"));
            //˰
            complexIncome.setExemptIncome((BigDecimal) map.get("t2"));
            //ҵ
            complexIncome.setCommercialHealthInsurance((BigDecimal) map.get("t13"));
            //˰ϱ
            complexIncome.setExtensionEndowmentInsurance((BigDecimal) map.get("t14"));
            //
            //complexIncome.setOther(BigDecimal.ZERO);
            //˰
            complexIncome.setTaxDeduction((BigDecimal) map.get("t17"));
            //ע
            //complexIncome.setRemark("");
            //
            // complexIncome.setDeductionAmount(BigDecimal.ZERO);
            //Ӧ˰
            //complexIncome.setPayableAmount(BigDecimal.ZERO);
            //˰
            //complexIncome.setTaxRate(BigDecimal.ZERO);
            //Ŀ*
            complexIncome.setIncomeItemName("һ񱨳");
            //۳˰
            complexIncome.setTaxDeductible((BigDecimal) map.get("t18"));
            laborRemunerationIncome.add(complexIncome);
        }
        return laborRemunerationIncome;
    }


    /**
     * ȫһԽб
     *
     * @return
     */
    private List<ComplexIncome> getAnnualOneTimeBonusIncomeList(
            Map<String, List<Map<String, Object>>> caluDataMap) {
        List<ComplexIncome> annualOneTimeBonusIncome = new ArrayList<>();
        List<Map<String, Object>> list = caluDataMap.get("qnycxjjsslb");
        for (int i = 0; list != null && i < list.size(); i++) {
            Map<String, Object> map = list.get(i);
            ComplexIncome complexIncome = new ComplexIncome();
            //*
            complexIncome.setName((String) map.get("personName"));
            //֤*
            complexIncome.setLicenseType((String) map.get("cardType"));
            //֤*
            complexIncome.setLicenseNumber((String) map.get("cardNumber"));
            //κ(һ¶ηн)
            //complexIncome.setBatchNo(0);
            //*
            complexIncome.setIncome((BigDecimal) map.get("t1"));
            //˰
            complexIncome.setExemptIncome((BigDecimal) map.get("t2"));
            //
            //complexIncome.setOther(BigDecimal.ZERO);
            //˰
            complexIncome.setTaxDeduction((BigDecimal) map.get("t17"));
            //ע
            //complexIncome.setRemark("");
            //Ŀ*
            complexIncome.setIncomeItemName("ȫһԽ");
            //׼۳ľ
            complexIncome.setDeductibleDonation((BigDecimal) map.get("t16"));
            annualOneTimeBonusIncome.add(complexIncome);
        }
        return annualOneTimeBonusIncome;
    }

    /**
     * ȡн
     *
     * @return
     */
    private NormalSalarySpecIncome getNormalSalarySpecIncome
    (Map<String, List<Map<String, Object>>> caluDataMap) {
        //н
        NormalSalarySpecIncome normalSalarySpecIncome = new NormalSalarySpecIncome();
        //ҵԱб*
        List<DetailCompanyEmployee> companyEmployeeList = getCompanyEmployeeList(caluDataMap);
        normalSalarySpecIncome.setCompanyEmployeeList(companyEmployeeList);
        //нǷҪר*
        //normalSalarySpecIncome.setZcgzxjsfxyzx("");
        //нǷۼƣ걨/¼ʱʹã
        //normalSalarySpecIncome.setZcgzxjsfcrlj("");
        //֧´רۼר˰нб˰Ч
        //normalSalarySpecIncome.setZcgzxjsfcrzx("");
        //Ƿ걨н˰
        //normalSalarySpecIncome.setDeclaredLastMonth("");
        //κ(һ¶ηн)
        //normalSalarySpecIncome.setBatchNo(0);
        //н(һ¶ηн)
        normalSalarySpecIncome.setImportLastMonthSalaryOption("");
        //ǷҪԱ걨
        normalSalarySpecIncome.setDeclareEmployeeOption("");
        //˰ѡ
        normalSalarySpecIncome.setAnnuityAndHouseProvidentFundOption("");
        //Ƿʹ´Ӧ˰
        normalSalarySpecIncome.setMultiCalculateTaxUseRefundTax("");
        //Ͻ
        normalSalarySpecIncome.setPersonalPensionOption("");
        //¸Ͻ
        normalSalarySpecIncome.setLastMonthPersonalPensionOption("");
        //ְڴ
        normalSalarySpecIncome.setResignDateStrategy("");
        //ǷʹѱԱϢвȫ
        //normalSalarySpecIncome.setSfsybsrybq();
        //н(ר)б
        //normalSalarySpecIncome.setLastMonthNormalSalary();
        List<ComplexIncome> normalSalarySpec = new ArrayList<>();
        //н(ר)б*
        normalSalarySpecIncome.setNormalSalarySpec(normalSalarySpec);
        List<Map<String, Object>> list = caluDataMap.get("zcgzxjlb");
        for (int i = 0; list != null && i < list.size(); i++) {
            Map<String, Object> map = list.get(i);
            ComplexIncome complexIncome = new ComplexIncome();
            //*
            complexIncome.setName((String) map.get("personName"));
            //֤*
            complexIncome.setLicenseType((String) map.get("cardType"));
            //֤*
            complexIncome.setLicenseNumber((String) map.get("cardNumber"));
            //Ƿϸ걨
            //complexIncome.setIsDetailedDeclaration("");
            //*
            complexIncome.setIncome((BigDecimal) map.get("t1"));
            //˰
            complexIncome.setExemptIncome((BigDecimal) map.get("t2"));
            //ϱ
            complexIncome.setEndowmentInsurance((BigDecimal) map.get("t3"));
            //ҽƱ
            complexIncome.setMedicalInsurance((BigDecimal) map.get("t4"));
            //ʧҵ
            complexIncome.setUnemploymentInsurance((BigDecimal) map.get("t5"));
            //ס
            complexIncome.setHouseProvidentFund((BigDecimal) map.get("t6"));
            //Ů֧
            complexIncome.setChildEducationExpenditure((BigDecimal) map.get("t64"));
            //֧
            complexIncome.setSupportElderExpenditure((BigDecimal) map.get("t65"));
            //ס֧
            complexIncome.setHouseRentExpenditure((BigDecimal) map.get("t67"));
            //ݴ֧
            complexIncome.setHouseLoanExpenditure((BigDecimal) map.get("t66"));
            //֧
            complexIncome.setContinueEducationExpenditure((BigDecimal) map.get("t68"));
            //Ӥ׶ջ֧
            complexIncome.setBabyCareExpenditure((BigDecimal) map.get("t70"));
            // ҵ/ְҵ
            complexIncome.setAnnuity((BigDecimal) map.get("t12"));
            //ҵ
            complexIncome.setCommercialHealthInsurance((BigDecimal) map.get("t13"));
            //˰ϱ
            complexIncome.setExtensionEndowmentInsurance((BigDecimal) map.get("t14"));
            //
            //complexIncome.setOther(BigDecimal.ZERO);
            normalSalarySpec.add(complexIncome);
        }
        return normalSalarySpecIncome;
    }

    /**
     * ȡҵԱб
     */
    private List<DetailCompanyEmployee> getCompanyEmployeeList(Map<String, List<Map<String, Object>>> caluDataMap) {
        List<DetailCompanyEmployee> companyEmployeeList = new ArrayList<>();
        List<Map<String, Object>> list = caluDataMap.get("zcgzxjlb");
        for (int i = 0; list != null && i < list.size(); i++) {
            Map<String, Object> map = list.get(i);
            DetailCompanyEmployee employee = new DetailCompanyEmployee();
            //*
            employee.setName((String) map.get("personName"));
            //֤*
            employee.setLicenseType((String) map.get("cardType"));
            //֤*
            employee.setLicenseNumber((String) map.get("cardNumber"));
            //ű
            //employee.setDeptNo("");
            //
            employee.setEmployeeNumber((String) map.get("personNumber"));
            //
            //employee.setPeriod("");
            //ֻ  йй״뾳ʱ䳬һ±
            employee.setPhone((String) map.get("phoneNumber"));
            //Ա״̬* ĬΪѡ/
            employee.setState((String) map.get("personstatusName"));
            //ǷԱ* ѡԱӪԱ֤ȯˡʵϰѧȫѧ
            employee.setIsEmployee((String) map.get("employedtypeName"));
            //ְȾҵ  ԱӪԱ֤ȯʱѡ񣺵״ְѧ״ְԱдĬΪ
            employee.setFirstEmploymentSituation((String) map.get("situationName"));
            //ܹ
            employee.setEmployedDate((String) map.get("employeddate"));
            //Ա*   /Ů
            employee.setGender((String) map.get("gender"));
            //* YYYY-MM-DD Ҫ֤ϱһ
            employee.setBirthday((String) map.get("birthday"));
            //*
            employee.setNationality((String) map.get("nationalityName"));
            //Ա* /
            employee.setArea((String) map.get("isoversea"));
            //ְ
            employee.setResignDate((String) map.get("departuredate"));
            //Ͷܶ
            employee.setPersonInvestment((BigDecimal) map.get("investmenttotal"));
            //Ͷʱ
            employee.setPersonInvestmentRatio((BigDecimal) map.get("investmentrate"));
            //Ƿм
            String isdisability = (String) map.get("isdisability");
            employee.setIsDisabled(isdisability);
            if ("".equals(isdisability)) {
                //todo м֤
                employee.setDisableCardTypeName("м֤");
            }
            //Ƿ
            employee.setIsMartyr((String) map.get("ishero"));
            //Ƿ
            employee.setIsBereavedGaffer((String) map.get("isold"));
            //м֤
            employee.setDisableCardNumber((String) map.get("discardnumber"));
            //֤
            employee.setMartyrCardNumber((String) map.get("herocardnumber"));
            //
            employee.setEmail((String) map.get("email"));
            //ѧ
            employee.setEducation((String) map.get("diploma"));
            //ְ
            employee.setPosition((String) map.get("positionName"));
            //ʡ()
            employee.setDepositBankProvinceName((String) map.get("bankcardprovince"));
            //
            employee.setDepositBank((String) map.get("bankcardprovince"));
            //˺
            employee.setBankAccount((String) map.get("account"));
            //סʡ
            employee.setResidentProvince((String) map.get("address"));
            //ס
            employee.setResidentCity((String) map.get("addresscity"));
            //ס
            employee.setResidentDistrict((String) map.get("addresscounty"));
            //סֵ
            employee.setResidentStreet((String) map.get("addressstreet"));
            //ʡ
            employee.setCensusProvince((String) map.get("hometown"));
            //
            employee.setCensusCity((String) map.get("hometowncity"));
            //
            employee.setCensusDistrict((String) map.get("hometowncountry"));
            //ֵ
            employee.setCensusStreet((String) map.get("hometownstreet"));
            //ϸַ
            employee.setCensusAddress((String) map.get("hometowndetailed"));
            //ע
            employee.setEmpRemark((String) map.get("desc"));
            //
            employee.setBirthplace((String) map.get("birthplaceName"));
            //״뾳ʱ
            employee.setFirstEntryDate((String) map.get("entrydate"));
            //Ԥ뾳ʱ
            employee.setEstimatedDepartureDate((String) map.get("leavedate"));
            //ϵʡ
            employee.setContactProvince((String) map.get("contactaddress"));
            //ϵس
            employee.setContactCity((String) map.get("contactcity"));
            //ϵ
            employee.setContactDistrict((String) map.get("contactcountry"));
            //ϵؽֵ
            employee.setContactStreet((String) map.get("contactstreet"));
            //ϵϸַ
            employee.setContactStreet((String) map.get("contactdetailed"));
            //
            employee.setChineseName((String) map.get("chinesename"));
            //˰
            employee.setTaxRelatedReason((String) map.get("taxreason"));
            //֤
            employee.setOtherLicenseNumber((String) map.get("othercardnumber"));
            //֤    ֤Ϊþ֤þ֤գл񹲺͹˹֤Aࣩл񹲺͹˹֤Bࣩл񹲺͹˹֤Cࣩ,л񹲺͹۰ľס֤,л񹲺͹̨ס֤йգ۰ľڵ̨֤ͨ½֤֤ͨ
            employee.setOtherLicenseType((String) map.get("othercardtype"));
            //Ƿ۳ãԭǷλ
            employee.setIsMainUnits((String) map.get("deductfees"));
            companyEmployeeList.add(employee);
        }
        return companyEmployeeList;
    }
}
